/*
 * Copyright (c) 2015 by inovat, innovative systeme - verkehr - tunnel - technik, Dipl.-Ing. H. C. Kniss
 *
 * This file is part of de.bsvrz.kex.tls.osi7.deausfallueberwachung.AuftragsListe
 *
 * de.bsvrz.kex.tls.osi7.deausfallueberwachung.AuftragsListe is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  de.bsvrz.kex.tls.osi7.deausfallueberwachung.AuftragsListe is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with de.bsvrz.kex.tls.osi7.deausfallueberwachung.AuftragsListe.  If not, see <http://www.gnu.org/licenses/>.
 *
 *  Diese Datei ist Teil von de.bsvrz.kex.tls.osi7.deausfallueberwachung.AuftragsListe.
 *
 *  de.bsvrz.kex.tls.osi7.deausfallueberwachung.AuftragsListe ist Freie Software: Sie können es unter den Bedingungen
 *  der GNU General Public License, wie von der Free Software Foundation,
 *  Version 3 der Lizenz oder (nach Ihrer Wahl) jeder späteren
 *  veröffentlichten Version, weiterverbreiten und/oder modifizieren.
 *
 *  de.bsvrz.kex.tls.osi7.deausfallueberwachung.AuftragsListe wird in der Hoffnung, dass es nützlich sein wird, aber
 *  OHNE JEDE GEWÄHRLEISTUNG, bereitgestellt; sogar ohne die implizite
 *  Gewährleistung der MARKTFÄHIGKEIT oder EIGNUNG FÜR EINEN BESTIMMTEN ZWECK.
 *  Siehe die GNU General Public License für weitere Details.
 *
 *  Sie sollten eine Kopie der GNU General Public License zusammen mit diesem
 *  Programm erhalten haben. Wenn nicht, siehe <http://www.gnu.org/licenses/>.
 *
 * Contact Information:
 * inovat, Dipl.-Ing. H. C. Kniss
 * An der Krautwiese 37
 * D-53783 Eitorf
 * +49 (0)2243 8464 193
 * info@inovat.de
 * www.inovat.de
 */

package de.bsvrz.kex.tls.osi7.deausfallueberwachung;


import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.List;

/**
 * Singletonklasse, welche die Auftragsliste für die DeAusfallUeberwachung verwaltet.
 * <p>
 * Die Auftragsliste ermöglicht das Hinzufügen von Aufträgen ({@link Auftrag} sowie die Entnahme des aktuell
 * auszuführenden Auftrags. Die Zugriffe auf die Auftragsliste ist intern syncronisiert.
 *
 * @author inovat, innovative systeme - verkehr - tunnel - technik
 * @author Dipl.-Ing. Hans Christian Kniß (HCK)
 * @version $Revision$ / $Date$ / ($Author$)
 */
public class AuftragsListe {
    private static AuftragsListe dieEinzigeInstanz = new AuftragsListe();
    private static final List<Auftrag> _listeAuftraege = new ArrayList<Auftrag>();

    //~ KONSTRUKTOREN  (und vom Konstruktor verwendete Methoden) ==============

    /**
     * Privater Konstruktor.
     */
    private AuftragsListe() {
    }

    //~ GET METHODEN ==========================================================

    /**
     * Liefert die Instanz des Singletons.
     *
     * @return Instanz des Singletons
     */
    public static AuftragsListe getInstanz() {
        return dieEinzigeInstanz;
    }

    //~ METHODEN ==============================================================

    /**
     * Fügt einen neuen Auftrag zur Liste der Aufträge hinzu. Die Liste wird intern so sortiert, dass die Methode {@link
     * #holeNaechstenAuftrag()} immer den zum aktuellen Zeitpunkt nächsten auszuführenden Auftrag zurückliefert.
     *
     * @param auftrag Neuer Auftrag, der in die Liste der Aufträge eingetragen werden soll.
     */
    public void addAuftrag(Auftrag auftrag) {
        synchronized (_listeAuftraege) {
            // Zuerst bisherigen (zeitlich alten) Auftrag fuer die DE-ATG-ASP-TYP Kombination entfernen...
            for(Auftrag auftragAlt:_listeAuftraege){
                if (auftragAlt.istAelterAls(auftrag)){
                    _listeAuftraege.remove(auftragAlt);
                    break;
                }
            }
            // .. und dann den zeitlich akuellen Auftrag eintragen 
            _listeAuftraege.add(auftrag);
            Collections.sort(_listeAuftraege, new Comparator<Auftrag>() {
                public int compare(Auftrag auftrag1, Auftrag auftrag2) {

                    // Zeitliche Sortierung der Aufträge nach Ausführungszeitpunkt
                    if (auftrag1.getAusfuehrungsZeitPunkt() < auftrag2.getAusfuehrungsZeitPunkt()) {
                        return -1;
                    }

                    if (auftrag1.getAusfuehrungsZeitPunkt() > auftrag2.getAusfuehrungsZeitPunkt()) {
                        return 1;
                    }

                    return 0;
                }
            });
        }
    }

    /**
     * Gibt zurück, ob noch Aufträge (aktuell oder zukünftig) anstehen.
     *
     * @return <code>true</code>, wenn noch Aufträge (aktuell oder zukünftig) anstehen, andernfalls <code>false</code>.
     */
    public boolean hatNochAuftraege() {
        synchronized (_listeAuftraege) {
            return _listeAuftraege.size() != 0;
        }
    }

    /**
     * Liefert eine Kopie der Auftragsliste, z.B. für Tests oder Debugausgaben des aktuellen Stands zu einem
     * bestimmten Zeitpunkt.
     *
     * @return Kopie der Auftragsliste.
     */
    public List<Auftrag> holeKopieAuftragsListe() {
        List<Auftrag> listeKopieAuftraege = new ArrayList<Auftrag>();

        synchronized (_listeAuftraege) {
            for (Auftrag auftrag : _listeAuftraege) {
                listeKopieAuftraege.add(auftrag);
            }
        }

        return listeKopieAuftraege;
    }

    /**
     * Liefert den, bezogen auf den aktuellen Zeitpunkt, nächsten auszuführenden Auftrag zurück.
     *
     * @return Nächster auszuführender Auftrag oder <code>null</code>, wenn kein Auftrag aktuell auszuführen ist. Der
     *         zurückgegebene Auftrag wird entfernt und kann nicht noch einmal abgerufen werden.
     */
    public Auftrag holeNaechstenAuftrag() {
        Auftrag naechsterAuftrag = null;

        synchronized (_listeAuftraege) {
            if ((_listeAuftraege.size() > 0) && (_listeAuftraege.get(0).getAusfuehrungsZeitPunkt() < System.currentTimeMillis())) {
                naechsterAuftrag = _listeAuftraege.get(0);
                _listeAuftraege.remove(0);
            }
        }

        return naechsterAuftrag;
    }
}
